#ifndef __solver_h
#define __solver_h




#include "rbfmatrix.h"
#include "precond.h"



namespace fmi
{
   

using namespace std;


// This class uses GMRES iteration to fit a radial basis function
// to a set of points
// Preconditioning is an option

// Instructions for using this class:
// Set the points and values.
// Call the functions SetMatrix and SetRightHandSideVector,
// then call Solve.


class solver
{

   public:


      typedef double          ScalarType;


      //void SetPointSet(PointSetType* ps);

      //const PointSetType* GetPointSet() const 
      //{ return m_PointSet; }



      //void SetPoints(const ScalarType* points);
      //void SetValues(const ScalarType* values);

      void SetPoints(const ScalarType* points, int numberOfPoints);
      void SetValues(const ScalarType* values, int numberOfPoints);


      // These functions are optional

      void PreconditioningOn() 
      { m_PreconditioningFlag = true;m_MatrixWasModified = true; }
      void PreconditioningOff() 
      { m_PreconditioningFlag = false;m_MatrixWasModified = true;}

      void SetMaxIterations(int i) { m_MaxIterations = i; }
      int GetMaxIterations() const { return m_MaxIterations; }

      void SetRestartIterations(int i) { m_RestartIterations = i; }
      int GetRestartIterations() const { return m_RestartIterations; }

      void SetTolerance(ScalarType t) { m_Tolerance = t; }
      ScalarType GetTolerance() const { return m_Tolerance; }
	
      void SetPolynomialDegree(int d);
      int GetPolynomialDegree() const { return m_PolynomialDegree; }
     
      void SetK(int i) { m_K = i;m_MatrixWasModified = true; } // K must be odd
      int GetK() const { return m_K; }
  
      void SetTau(ScalarType t) { m_Tau = t;m_MatrixWasModified = true; }
      ScalarType GetTau() const { return m_Tau; }

      void SetLambda(ScalarType lam) {m_Lambda = lam;m_MatrixWasModified = true;}
      ScalarType GetLambda() const { return m_Lambda; }

	
      int GetNumberOfPoints() const { return 3*m_Points.size(); }

      
      // Call this after calling the above functions
      void Update();

      // Call this after calling Solve to retrieve the solution
      // returns a const pointer to the beginning of the solution vector
      const ScalarType* GetSolution() const { return &m_x[0]; }

      // return the number of rows (or columns) in the matrix
      int GetMatrixSize() const { return 3*m_Points.size()+
				     m_NumberPolynomialTerms;}

      solver();
      ~solver() {}

 private:


      typedef Preconditioner  PreconditionerType;

   
      typedef PreconditionerType::VectorType VectorType;
      typedef PreconditionerType::MatrixType MatrixType;
      MatrixType H;


      PreconditionerType pre;

      //typedef pointset PointSetType;

      //PointSetType* m_PointSet;
      //PointSetType m_PointSet;

      //typedef std::valarray< ScalarType > VectorType;
      VectorType m_Points;
      VectorType m_Values;

      bool m_PreconditioningFlag;

      rbfmatrix m_A;
      VectorType m_b;
      VectorType m_x;


      int m_MaxIterations;
      int m_RestartIterations;

      ScalarType m_Tolerance;

      int m_PolynomialDegree;
      int m_NumberPolynomialTerms;

      int m_K;
      ScalarType m_Tau;
      ScalarType m_Lambda;

      bool m_MatrixWasModified;
      bool m_RightHandSideWasModified;

      solver(const solver& rhs); // Purposely not implemented
      void operator=(const solver& rhs); // Purposely not implemented

};



}


#endif
